"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard")["default"];
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault")["default"];
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCacheConfig = exports["default"] = void 0;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/objectSpread2"));
var _react = _interopRequireWildcard(require("react"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _shim = require("use-sync-external-store/shim");
var _utils = require("@lilib/utils");
var GLOBAL_CACHE_ID = "__LILIB_HOOKS_GLOBAL_CACHE__";
var defaultValue = {
  global: false,
  cacheTime: 5 * 60 * 1000,
  cacheSync: false
};
var CacheContext = /*#__PURE__*/(0, _react.createContext)((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, defaultValue), {}, {
  cache: new _utils.MemoryCache()
}));
var useCacheConfig = function useCacheConfig() {
  return (0, _react.useContext)(CacheContext);
};
exports.useCacheConfig = useCacheConfig;
var CacheConfig = function CacheConfig(props) {
  var config = useCacheConfig();
  var cacheRef = (0, _react.useRef)();
  var value = (0, _omit["default"])(props, "inherit", "children");
  if (props.inherit) {
    value = (0, _utils.mergeWithDefined)(config, value);
  } else {
    value = (0, _utils.mergeWithDefined)(defaultValue, value);
  }
  if (value.global && _utils.inBrowser && window[GLOBAL_CACHE_ID]) {
    value.cache = window[GLOBAL_CACHE_ID];
  }
  if (!value.cache) {
    if (!cacheRef.current) {
      cacheRef.current = new _utils.MemoryCache();
    }
    value.cache = cacheRef.current;
  }
  if (value.global && _utils.inBrowser && !window[GLOBAL_CACHE_ID]) {
    window[GLOBAL_CACHE_ID] = value.cache;
  }
  var cache = value.cache;
  var subscribe = (0, _react.useCallback)(function (listener) {
    if (!cache.isReady()) {
      cache.once("ready", listener);
      return function () {
        cache.off("ready", listener);
      };
    } else {
      return function () {};
    }
  }, [cache]);
  var getSnapshot = (0, _react.useCallback)(function () {
    return cache.isReady();
  }, [cache]);
  var ready = (0, _shim.useSyncExternalStore)(subscribe, getSnapshot);
  (0, _react.useDebugValue)(ready ? "Ready" : "Unready");
  (0, _react.useDebugValue)(value);
  return /*#__PURE__*/_react["default"].createElement(CacheContext.Provider, {
    value: value
  }, ready ? props.children : value.fallback);
};
CacheConfig.Context = CacheContext;
CacheConfig.useConfig = useCacheConfig;
var _default = CacheConfig;
exports["default"] = _default;