"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault")["default"];
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/slicedToArray"));
var _react = require("react");
var _isEqual = _interopRequireDefault(require("lodash/isEqual"));
var _isFunction = _interopRequireDefault(require("lodash/isFunction"));
var _useUpdate = _interopRequireDefault(require("../useUpdate"));
var _usePersist = _interopRequireDefault(require("../usePersist"));
var _useInterval3 = _interopRequireDefault(require("../useInterval"));
var _useSafeState3 = _interopRequireDefault(require("../useSafeState"));
var _useEventListener = _interopRequireDefault(require("../useEventListener"));
var _useMemoizedValue = _interopRequireDefault(require("../useMemoizedValue"));
function createStorageHook(storage) {
  function useStorage(key, options) {
    var _ref = options || {},
      defaultValueOption = _ref.defaultValue,
      _ref$polling = _ref.polling,
      polling = _ref$polling === void 0 ? false : _ref$polling,
      _ref$pollingInterval = _ref.pollingInterval,
      pollingInterval = _ref$pollingInterval === void 0 ? 100 : _ref$pollingInterval,
      _ref$compare = _ref.compare,
      compare = _ref$compare === void 0 ? _isEqual["default"] : _ref$compare,
      validate = _ref.validate,
      _ref$serialize = _ref.serialize,
      serialize = _ref$serialize === void 0 ? JSON.stringify : _ref$serialize,
      _ref$deserialize = _ref.deserialize,
      deserialize = _ref$deserialize === void 0 ? JSON.parse : _ref$deserialize;
    var defaultValue = (0, _useMemoizedValue["default"])(defaultValueOption, compare);
    function getStoredValue() {
      try {
        var _raw = storage === null || storage === void 0 ? void 0 : storage.getItem(key);
        if (_raw != null) {
          var _value = deserialize(_raw);
          if (validate) {
            return validate(_value) ? _value : defaultValue;
          } else {
            return _value;
          }
        }
      } catch (error) {
        console.error(error);
      }
      return defaultValue;
    }
    var _useSafeState = (0, _useSafeState3["default"])(getStoredValue),
      _useSafeState2 = (0, _slicedToArray2["default"])(_useSafeState, 2),
      value = _useSafeState2[0],
      setValue = _useSafeState2[1];
    var setStorage = (0, _usePersist["default"])(function (newValue) {
      setValue(function (prevValue) {
        newValue = (0, _isFunction["default"])(newValue) ? newValue(prevValue) : newValue;
        if (newValue === undefined) {
          storage === null || storage === void 0 ? void 0 : storage.removeItem(key);
          return defaultValue;
        }
        try {
          storage === null || storage === void 0 ? void 0 : storage.setItem(key, serialize(newValue));
        } catch (error) {
          console.error(error);
        }
        return newValue;
      });
    });
    var handleChange = function handleChange() {
      var newValue = getStoredValue();
      if (!compare(newValue, value)) {
        setValue(newValue);
      }
    };
    var _useInterval = (0, _useInterval3["default"])(handleChange, pollingInterval),
      _useInterval2 = (0, _slicedToArray2["default"])(_useInterval, 2),
      startPolling = _useInterval2[0],
      cancelPolling = _useInterval2[1];
    (0, _react.useEffect)(function () {
      cancelPolling();
      if (polling) {
        startPolling();
      }
    }, [polling] // eslint-disable-line
    );

    (0, _useEventListener["default"])(function () {
      return window;
    }, "storage", handleChange);
    (0, _useUpdate["default"])(function () {
      cancelPolling();
      setValue(getStoredValue());
      if (polling) {
        startPolling();
      }
    }, [key]);
    (0, _useUpdate["default"])(function () {
      if ((storage === null || storage === void 0 ? void 0 : storage.getItem(key)) == null && !compare(defaultValue, value)) {
        setValue(defaultValue);
      }
    }, [defaultValue]);
    (0, _react.useDebugValue)(value);
    (0, _react.useDebugValue)(key, function (key) {
      return "(raw) " + (storage === null || storage === void 0 ? void 0 : storage.getItem(key));
    });
    return [value, setStorage];
  }
  return useStorage;
}
var _default = createStorageHook;
exports["default"] = _default;