"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault")["default"];
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _react = require("react");
var _raf = _interopRequireDefault(require("raf"));
var _isNumber = _interopRequireDefault(require("lodash/isNumber"));
var _utils = require("@lilib/utils");
var _useUnmount = _interopRequireDefault(require("./useUnmount"));
var _usePersist = _interopRequireDefault(require("./usePersist"));
var _useLatestRef = _interopRequireDefault(require("./useLatestRef"));
function useAnimation(callback, options) {
  var duration = 0;
  var algorithm;
  if ((0, _isNumber["default"])(options)) {
    duration = options;
  } else if (options) {
    if (options.duration) {
      duration = +options.duration;
    }
    if (options.algorithm) {
      algorithm = options.algorithm;
    }
  }
  if (process.env.NODE_ENV !== "production") {
    (0, _utils.warning)(!(0, _isNumber["default"])(duration) || !(duration > 0), "You should provide a positive number as the duration.", {
      scope: "useAnimation"
    });
  }
  var timerRef = (0, _react.useRef)(0);
  var startTimeRef = (0, _react.useRef)(0);
  var callbackRef = (0, _useLatestRef["default"])(callback);
  var step = (0, _usePersist["default"])(function (timestamp) {
    if (!startTimeRef.current) {
      startTimeRef.current = timestamp;
    }
    var elapsed = timestamp - startTimeRef.current;
    var percent = duration > 0 ? elapsed / duration : 1;
    if (percent < 0) percent = 0;
    if (percent > 1) percent = 1;
    if (percent < 1) {
      timerRef.current = (0, _raf["default"])(step);
    }
    if (algorithm) {
      callbackRef.current(algorithm(percent));
    } else {
      callbackRef.current(percent);
    }
  });
  var cancel = (0, _react.useCallback)(function () {
    if (timerRef.current) {
      _raf["default"].cancel(timerRef.current);
      timerRef.current = 0;
    }
    startTimeRef.current = 0;
  }, []);
  var start = (0, _usePersist["default"])(function () {
    cancel();
    timerRef.current = (0, _raf["default"])(step);
  });
  (0, _useUnmount["default"])(cancel);
  return [start, cancel];
}
var _default = useAnimation;
exports["default"] = _default;