"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault")["default"];
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/slicedToArray"));
var _react = require("react");
var _utils = require("@lilib/utils");
var _useUpdate = _interopRequireDefault(require("./useUpdate"));
var _usePersist = _interopRequireDefault(require("./usePersist"));
var _useSafeState3 = _interopRequireDefault(require("./useSafeState"));
var _CacheConfig = require("./configs/CacheConfig");
function useCache(key) {
  var _options$cache, _options$cacheTime, _options$cacheSync;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var config = (0, _CacheConfig.useCacheConfig)();
  var defaultValue = options.defaultValue,
    validate = options.validate,
    onSet = options.onSet,
    onDelete = options.onDelete;
  var cache = (_options$cache = options.cache) !== null && _options$cache !== void 0 ? _options$cache : config.cache;
  var cacheTime = (_options$cacheTime = options.cacheTime) !== null && _options$cacheTime !== void 0 ? _options$cacheTime : config.cacheTime;
  var cacheSync = (_options$cacheSync = options.cacheSync) !== null && _options$cacheSync !== void 0 ? _options$cacheSync : config.cacheSync;
  var cacheSyncOnSet = false;
  var cacheSyncOnDelete = false;
  if (cacheSync === true) {
    cacheSyncOnSet = true;
    cacheSyncOnDelete = true;
  } else if (cacheSync) {
    cacheSyncOnSet = !!cacheSync.set;
    cacheSyncOnDelete = !!cacheSync["delete"];
  }
  var getCachedData = function getCachedData() {
    var cachedData = cache.get(key);
    if (cachedData !== undefined) {
      if (validate) {
        return validate(cachedData) ? cachedData : defaultValue;
      } else {
        return cachedData;
      }
    } else {
      return defaultValue;
    }
  };
  var _useSafeState = (0, _useSafeState3["default"])(getCachedData),
    _useSafeState2 = (0, _slicedToArray2["default"])(_useSafeState, 2),
    value = _useSafeState2[0],
    setValue = _useSafeState2[1];
  var setCache = (0, _usePersist["default"])(function (newValue) {
    if (newValue === undefined) {
      cache["delete"](key);
    } else {
      cache.set(key, newValue, {
        cacheTime: cacheTime
      });
    }
  });
  var shouldHandleSet = !!onSet || cacheSyncOnSet;
  var shouldHandleDelete = !!onDelete || cacheSyncOnDelete;
  var handleSet = (0, _usePersist["default"])(function (data) {
    if (cacheSyncOnSet) {
      setValue(getCachedData());
    }
    if (onSet) {
      onSet(data);
    }
  });
  var handleDelete = (0, _usePersist["default"])(function (data) {
    if (cacheSyncOnDelete) {
      setValue(getCachedData());
    }
    if (onDelete) {
      onDelete(data);
    }
  });
  (0, _react.useEffect)(function () {
    if (shouldHandleSet) {
      cache["for"](key).on("set", handleSet);
      return function () {
        cache["for"](key).off("set", handleSet);
      };
    }
  }, [key, cache, shouldHandleSet, handleSet]);
  (0, _react.useEffect)(function () {
    if (shouldHandleDelete) {
      cache["for"](key).on("delete", handleDelete);
      return function () {
        cache["for"](key).off("delete", handleDelete);
      };
    }
  }, [key, cache, shouldHandleDelete, handleDelete]);
  (0, _useUpdate["default"])(function () {
    setValue(getCachedData());
  }, [key, cache]);
  (0, _useUpdate["default"])(function () {
    if (process.env.NODE_ENV !== "production") {
      (0, _utils.warning)(true, "You should provide an immutable value for the `key` option.", {
        scope: "useCache"
      });
    }
  }, [key]);
  (0, _react.useDebugValue)(value);
  return [value, setCache];
}
var _default = useCache;
exports["default"] = _default;