import { useCallback, useRef } from "react";
import raf from "raf";
import isNumber from "lodash/isNumber";
import { warning } from "@lilib/utils";
import useUnmount from "./useUnmount";
import usePersist from "./usePersist";
import useLatestRef from "./useLatestRef";
function useAnimation(callback, options) {
  var duration = 0;
  var algorithm;
  if (isNumber(options)) {
    duration = options;
  } else if (options) {
    if (options.duration) {
      duration = +options.duration;
    }
    if (options.algorithm) {
      algorithm = options.algorithm;
    }
  }
  if (process.env.NODE_ENV !== "production") {
    warning(!isNumber(duration) || !(duration > 0), "You should provide a positive number as the duration.", {
      scope: "useAnimation"
    });
  }
  var timerRef = useRef(0);
  var startTimeRef = useRef(0);
  var callbackRef = useLatestRef(callback);
  var step = usePersist(function (timestamp) {
    if (!startTimeRef.current) {
      startTimeRef.current = timestamp;
    }
    var elapsed = timestamp - startTimeRef.current;
    var percent = duration > 0 ? elapsed / duration : 1;
    if (percent < 0) percent = 0;
    if (percent > 1) percent = 1;
    if (percent < 1) {
      timerRef.current = raf(step);
    }
    if (algorithm) {
      callbackRef.current(algorithm(percent));
    } else {
      callbackRef.current(percent);
    }
  });
  var cancel = useCallback(function () {
    if (timerRef.current) {
      raf.cancel(timerRef.current);
      timerRef.current = 0;
    }
    startTimeRef.current = 0;
  }, []);
  var start = usePersist(function () {
    cancel();
    timerRef.current = raf(step);
  });
  useUnmount(cancel);
  return [start, cancel];
}
export default useAnimation;