import { useMemo, useRef } from "react";
import isObject from "lodash/isObject";
import throttle from "lodash/throttle";
import usePersist from "./usePersist";
import useUnmount from "./useUnmount";
import useLatestRef from "./useLatestRef";
function useThrottle(callback, options) {
  var wait = 0;
  var leading = false;
  var trailing = true;
  if (typeof options === "number") {
    wait = options;
  } else if (isObject(options)) {
    if (options.wait !== undefined) {
      wait = +options.wait;
    }
    if (options.leading !== undefined) {
      leading = !!options.leading;
    }
    if (options.trailing !== undefined) {
      trailing = !!options.trailing;
    }
  }
  var callbackRef = useLatestRef(callback);
  var throttledRef = useRef();
  useMemo(function () {
    var _throttledRef$current;
    (_throttledRef$current = throttledRef.current) === null || _throttledRef$current === void 0 ? void 0 : _throttledRef$current.flush();
    throttledRef.current = throttle(function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      // @ts-ignore
      return callbackRef.current.apply(this, args);
    }, wait, {
      leading: leading,
      trailing: trailing
    });
  }, [wait, leading, trailing] // eslint-disable-line
  );

  var throttled = usePersist(function () {
    var _throttledRef$current2;
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }
    // @ts-ignore
    return (_throttledRef$current2 = throttledRef.current) === null || _throttledRef$current2 === void 0 ? void 0 : _throttledRef$current2.apply(this, args);
  });
  var flush = usePersist(function () {
    var _throttledRef$current3;
    return (_throttledRef$current3 = throttledRef.current) === null || _throttledRef$current3 === void 0 ? void 0 : _throttledRef$current3.flush();
  });
  var cancel = usePersist(function () {
    var _throttledRef$current4;
    (_throttledRef$current4 = throttledRef.current) === null || _throttledRef$current4 === void 0 ? void 0 : _throttledRef$current4.cancel();
  });
  useUnmount(cancel);
  return [throttled, {
    flush: flush,
    cancel: cancel
  }];
}
export default useThrottle;