import React, { FC, ReactNode } from "react";
export interface LoadStoreInterface {
    isLoading(key: any, loader?: Function): boolean;
    setLoading(key: any, loader: Function): any;
    deleteLoading(key: any, loader?: Function): any;
    getReloaders(key: any): Function[];
    addReloader(key: any, reloader: Function): any;
    removeReloader(key: any, reloader: Function): any;
    for(key: any): {
        on(name: "cancel", listener: () => void): any;
        on(name: "success", listener: (data: any) => void): any;
        on(name: "failure", listener: (error: any) => void): any;
        off(name: "cancel", listener: () => void): any;
        off(name: "success", listener: (data: any) => void): any;
        off(name: "failure", listener: (error: any) => void): any;
        emit(name: "cancel"): any;
        emit(name: "success", data: any): any;
        emit(name: "failure", error: any): any;
    };
}
export interface LoadSharedOptions {
    cacheTime?: number;
    staleTime?: number;
    retry?: boolean;
    retryLimit?: number;
    retryInterval?: number | ((count: number) => number);
    fallbackRetry?: boolean;
    fallbackRetryLimit?: number;
    fallbackRetryInterval?: number | ((count: number) => number);
    polling?: boolean;
    pollingInterval?: number;
    pollingInPageHiding?: boolean;
    pollingIntervalInPageHiding?: number;
    autoReloadWaitTime?: number;
    autoReloadOnPageShow?: boolean;
    autoReloadOnWindowFocus?: boolean;
    autoReloadOnNetworkReconnect?: boolean;
}
export interface LoadConfigValue extends Required<LoadSharedOptions> {
    global: boolean;
    store: LoadStoreInterface;
    onSuccess?: (data: any, key?: {}) => void;
    onFailure?: (error: any, key?: {}) => void;
    onFinally?: (key?: {}) => void;
    handleSuccess?: (data: any, key?: {}) => void;
    handleFailure?: (error: any, key?: {}) => void;
    handleFinally?: (key?: {}) => void;
}
export interface LoadConfigProps extends Omit<Partial<LoadConfigValue>, "store"> {
    inherit?: boolean;
    children?: ReactNode;
}
declare const LoadContext: React.Context<LoadConfigValue>;
export declare const useLoadConfig: () => LoadConfigValue;
declare const LoadConfig: FC<LoadConfigProps> & {
    Context: typeof LoadContext;
    useConfig: typeof useLoadConfig;
};
export default LoadConfig;
