import "requestidlecallback";
import { DependencyList } from "react";
import { PromiseResolve } from "@lilib/utils";
import { LoadSharedOptions } from "./configs/LoadConfig";
export declare type LoadCallback = (...args: any[]) => Promise<any>;
export declare type LoadData<T extends LoadCallback> = PromiseResolve<ReturnType<T>>;
export interface LoadHookOptions<Callback extends LoadCallback> extends LoadSharedOptions {
    key?: {};
    idle?: boolean | IdleRequestOptions;
    imperative?: boolean;
    independent?: boolean;
    fallback?: Callback;
    initialData?: LoadData<Callback>;
    defaultParams?: Parameters<Callback>;
    cacheKey?: {};
    cacheValidate?: (data: any) => boolean;
    onSuccess?: (data: LoadData<Callback>) => void;
    onFailure?: (error: any) => void;
    onFinally?: () => void;
}
interface Results<Callback extends LoadCallback> {
    data: LoadData<Callback> | undefined;
    error: any;
    loading: boolean;
    reloading: boolean;
    initializing: boolean;
    load: (...params: Parameters<Callback>) => ReturnType<Callback>;
    force: (...params: Parameters<Callback>) => ReturnType<Callback>;
    reload: (options?: {
        force?: boolean;
    }) => void;
    update: (newData: LoadData<Callback> | ((prevData?: LoadData<Callback>) => LoadData<Callback>)) => void;
    cancel: () => void;
}
declare function useLoad<Callback extends LoadCallback>(callback: Callback, deps: DependencyList, options: LoadHookOptions<Callback> & {
    initialData: LoadData<Callback>;
}): Readonly<Results<Callback> & {
    data: LoadData<Callback>;
}>;
declare function useLoad<Callback extends LoadCallback>(callback: Callback, deps?: DependencyList, options?: LoadHookOptions<Callback>): Readonly<Results<Callback>>;
export default useLoad;
